<?php

App::uses('DiscountType', 'EvDiscount.Model');

class Discount extends EvDiscountAppModel {

	public $displayField = 'code';

	public $belongsTo = array(
		'DiscountType' => array(
			'className' => 'EvDiscount.DiscountType'
		)
	);


	public $validate = array(
		'code' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Required'
			),
			'unique' => array(
				'rule' => 'isUnique',
				'message' => 'Discount codes must be unique'
			)
		),
		'discount_type_id' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Required'
			)
		),
		'amount' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Required'
			)
		),
		'min_order' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Required'
			)
		),
		'max_uses' => array(
			'required' => array(
				'rule' => 'notEmpty',
				'message' => 'Required'
			)
		)
	);


/**
 * Construct and add in virtual fields
 */
	public function __construct($id = false, $table = null, $ds = null) {

		parent::__construct($id, $table, $ds);

		// Work out the remaining uses of a discount code for convinience.
		$this->virtualFields['remaining_uses'] = $this->escapeField('max_uses') . ' - ' . $this->escapeField('uses');

	}


	public function calculateDiscount($subtotal, $discountCode) {

		$now = date('Y-m-d H:i:s');

		$discount = $this->find(
			'first',
			array(
				'conditions' => array(
					'Discount.code' => $discountCode,
					'Discount.min_order <=' => $subtotal,
					array(
						'OR' => array(
							'Discount.start_date' => null,
							'Discount.start_date <=' => $now
						)
					),
					array(
						'OR' => array(
							'Discount.end_date' => null,
							'Discount.end_date >=' => $now
						)
					),
					'Discount.remaining_uses >' => 0,
					'Discount.is_active' => true
				)
			)
		);

		$discountTotal = 0;

		if (!empty($discount)) {

			$discountTotal = 0;

			if ($discount['Discount']['discount_type_id'] == DiscountType::PERCENTAGE) {

				$discountTotal = $subtotal * $discount['Discount']['amount'] / 100;

			} elseif ($discount['Discount']['discount_type_id'] == DiscountType::FIXED) {

				// Discount can never be more than the total payable.
				$discountTotal = $discount['Discount']['amount']>$subtotal ? $subtotal : $discount['Discount']['amount'];

			}

		}

		return $discountTotal;

	}
	
}